import com.atlassian.jira.component.ComponentAccessor
import com.atlassian.jira.issue.search.SearchRequestManager
import com.atlassian.query.clause.*
import com.midori.jira.plugin.pdfview.model.ColumnConfiguration

if (!binding.variables.containsKey("columnConfiguration")) {
    columnConfiguration = null
}
if (!binding.variables.containsKey("automationTriggerJiraJqlExpression")) {
	automationTriggerJiraJqlExpression = null
}
if (!binding.variables.containsKey("searchRequest")) {
	searchRequest = null
}

issueNavigator = new IssueNavigatorTool(
	user: user,
	defaultSearchRequest: searchRequest,
	tableLayoutFactory: tableLayoutFactory,
	columnLayoutManager: columnLayoutManager,
	columnConfiguration: columnConfiguration,
	automationTriggerJiraJqlExpression: automationTriggerJiraJqlExpression
)

/**
 * Tool to export dynamic column configurations from Issue Navigator.
 */
public class IssueNavigatorTool {
	def DEFAULT_COLUMNS = [ "issuetype", "issuekey", "summary", "priority", "status", "assignee", "reporter", "updated" ]

	def searchRequestManager = ComponentAccessor.getOSGiComponentInstanceOfType(SearchRequestManager.class)
	def tableLayoutFactory // cannot be obtained from ComponentAccessor

	def user
	def defaultSearchRequest

	def columnLayoutManager
	def columnConfiguration
	def issueTableLayout

	def automationTriggerJiraJqlExpression

	def getSearchRequest() {
		def searchRequest
		// if automation is used, find the search request (saved filter) object based on the automation trigger's JQL
		def jql = automationTriggerJiraJqlExpression
		if (jql != null) {
			def filterIdOrName = parseFilterIdOrNameFromJql(jql)
			if (filterIdOrName != null) {
				if (filterIdOrName.isInteger()) {
					// find by ID
					searchRequest = searchRequestManager.getSearchRequestById(Long.parseLong(filterIdOrName))
				}
				if (searchRequest == null) {
					// find by name
					def searchRequestsFound = searchRequestManager.findByNameIgnoreCase(filterIdOrName)
					if (!searchRequestsFound.empty) {
						searchRequest = searchRequestsFound.get(0)
					}
				}
			}
		}

		return (searchRequest != null) ? searchRequest : defaultSearchRequest
	}

	def getIssueTableLayout(searchRequest) {
		if (searchRequest != null) {
			// get the filter's column layout
			if (columnConfiguration == null || columnConfiguration == ColumnConfiguration.FILTER) {
				issueTableLayout = tableLayoutFactory.getStandardExcelLayout(searchRequest, user)
			} else {
				def columnLayout = (columnConfiguration == ColumnConfiguration.SYSTEM) ? columnLayoutManager.getDefaultColumnLayout(user) : columnLayoutManager.getColumnLayout(user)
				issueTableLayout = [ columns: columnLayout.getColumnLayoutItems() ]
			}
		} else {
			// get a sensible default column layout
			issueTableLayout = tableLayoutFactory.getDashboardLayout(user, DEFAULT_COLUMNS)
		}

		return issueTableLayout
	}

	/**
	 * Parses a saved filter's name or ID from a JQL query string (in the format: 'filter = 12' or 'filter = "My open bugs"').
	 * @return filter id or name without quotation marks or apostrophes, or null if wasn't found.
	 */
	private parseFilterIdOrNameFromJql(String jql) {
		// parse JQL
		def clazz = ComponentAccessor.class.classLoader.loadClass("com.atlassian.jira.jql.parser.JqlQueryParser")
		def jqlQueryParser = ComponentAccessor.getOSGiComponentInstanceOfType(clazz)
		def query = jqlQueryParser.parseQuery(jql)

		// search for the "filter =" clause
		def filterIdOrName = query.whereClause.accept(new ClauseVisitor<String>() {
			String visit(AndClause andClause) {
				visitSubClauses(andClause.clauses)
			}

			String visit(NotClause notClause) {
				visitSubClauses(notClause.clauses)
			}

			String visit(OrClause orClause) {
				visitSubClauses(orClause.clauses)
			}

			String visit(TerminalClause clause) {
				return (("filter".equals(clause.name.toLowerCase())) && ("=".equals(clause.operator.displayString))) ? ((clause.operand.longValue != null) ? clause.operand.longValue : clause.operand.stringValue) : null
			}

			String visit(WasClause clause) {
				return null
			}

			String visit(ChangedClause clause) {
				return null
			}

			private String visitSubClauses(def clauses) {
				for(clause in clauses) {
					def filterIdOrName = visit(clause)
					if(filterIdOrName != null) {
						return filterIdOrName
					}
				}
				return null
			}
		})

		return filterIdOrName
	}
}
