import com.atlassian.jira.component.ComponentAccessor
import org.apache.log4j.Logger

serviceDesk = new ServiceDeskTool()

/**
 * Velocity tool to get Jira Service Desk ticket data.
 */
class ServiceDeskTool {
	def log = Logger.getLogger(this.getClass())

	/* Jira Service Desk API. */
	def approvalServiceInitialized = false
	def approvalService

	/**
	 * Returns the decisions for an approval (as a map from the approver's display name to the decision).
	 */
	def getApprovalDecisions(def user, def approval) {
		// lazily load dependency (attempt only once)
		if (!approvalServiceInitialized) {
			try {
				def approvalServiceClass = Class.forName("com.atlassian.servicedesk.api.approval.ApprovalService")
				approvalService = ComponentAccessor.getOSGiComponentInstanceOfType(approvalServiceClass)
			} catch (Exception ex) {
				log.warn("Failed to find the ApprovalService", ex)
			}
			approvalServiceInitialized = true
		}

		def decisions = [:]
		def approvers = getApprovers(user, approval)
		if (approvers != null) {
			approvers.each {
				decisions.put(it.approverUser.displayName, it.approverDecision)
			}
		} else {
			log.warn("Failed to get approvers")
		}

		return decisions
	}

	private def getApprovers(def user, def approval) {
		if (!approvalService) {
			return null
		}
		def approvers = approvalService.getApprovers(user, approval)
		if (approvers.metaClass.respondsTo(approvers, "isRight")) {
			return approvers.isRight() ? approvers.right().get() : null
		}
		return approvers
	}
}
